@@ -244,13 +244,16 @@ class ObjCLazyModule(ModuleType):
             for nm, tp in re.findall(
                 r"\$([A-Z0-9a-z_]*)(@[^$]*)?(?=\$)", self.__varmap
             ):
+                if not nm:
+                    continue  # XXX: needed for libdispatch, likely bug
                 if tp and tp.startswith("@="):
                     specials.append((nm, tp[2:]))
                 else:
                     varmap.append((nm, b"@" if not tp else tp[1:].encode("ascii")))
 
             dct = {}
-            objc.loadBundleVariables(self.__bundle, dct, varmap)
+            if varmap:  # XXX: See XXX above
+                objc.loadBundleVariables(self.__bundle, dct, varmap)
 
             for nm in dct:
                 if nm not in self.__dict__:
@@ -365,7 +368,7 @@ class ObjCLazyModule(ModuleType):
                 return result
 
         if self.__varmap:
-            m = re.search(fr"\${name}(@[^$]*)?\$", self.__varmap)
+            m = re.search(rf"\${name}(@[^$]*)?\$", self.__varmap)
             if m is not None:
                 tp = m.group(1)
                 if not tp:
@@ -391,7 +394,7 @@ class ObjCLazyModule(ModuleType):
                 return result
 
         if self.__enummap:
-            m = re.search(fr"\${name}@([^$]*)\$", self.__enummap)
+            m = re.search(rf"\${name}@([^$]*)\$", self.__enummap)
             if m is not None:
                 result = self.__prs_enum(m.group(1))
                 if name in self.__enum_deprecated:
