@@ -19,40 +19,23 @@
 #import <Foundation/NSDictionary.h>
 #import <Foundation/NSEnumerator.h>
 #import <Foundation/NSInvocation.h>
+#import <Foundation/NSKeyValueObserving.h>
 #import <Foundation/NSMethodSignature.h>
 #import <Foundation/NSObject.h>
 #import <Foundation/NSString.h>
 
-#if MAC_OS_X_VERSION_MAX_ALLOWED >= MAC_OS_X_VERSION_10_3
-#import <Foundation/NSKeyValueObserving.h>
-#endif
-
 #import "OC_PythonUnicode.h"
 
 NS_ASSUME_NONNULL_BEGIN
 
-/* XXX: Is this still needed? */
-extern NSString* const NSUnknownKeyException; /* Radar #3336042 */
-
 @implementation OC_PythonObject
 + (id<NSObject> _Nullable)objectWithPythonObject:(PyObject*)obj
 {
-    id instance;
-    if (likely(PyObjCObject_Check(obj))) {
-        instance = PyObjCObject_GetObject(obj);
-    } else {
-        instance = [[[self alloc] initWithPyObject:obj] autorelease];
-    }
-    return instance;
+    return [[[self alloc] initWithPyObject:obj] autorelease];
 }
 
 - (id _Nullable)initWithPyObject:(PyObject*)obj
 {
-    /* XXX: Why check for NULL here? */
-    if (pyObject) {
-        PyObjC_UnregisterObjCProxy(pyObject, self);
-    }
-
     PyObjC_RegisterObjCProxy(obj, self);
 
     SET_FIELD_INCREF(pyObject, obj);
@@ -63,27 +46,37 @@ extern NSString* const NSUnknownKeyException; /* Radar #3336042 */
 - (oneway void)release
 {
     /* See comment in OC_PythonUnicode */
-    if (unlikely(!Py_IsInitialized())) {
+    if (unlikely(!Py_IsInitialized())) { // LCOV_BR_EXCL_LINE
+        // LCOV_EXCL_START
         [super release];
         return;
+        // LCOV_EXCL_STOP
     }
 
     PyObjC_BEGIN_WITH_GIL
         @try {
             [super release];
 
-        } @catch (NSObject* exc) {
+        } @catch (NSObject* exc) { // LCOV_BR_EXCL_LINE
+            /* This catch statement is here mostly
+             * for code consistency, [NSProxy release]
+             * should never raise.
+             */
+            // LCOV_EXCL_START
             PyObjC_LEAVE_GIL;
             @throw;
+            // LCOV_EXCL_STOP
         }
     PyObjC_END_WITH_GIL
 }
 
 - (void)dealloc
 {
-    if (unlikely(!Py_IsInitialized())) {
+    if (unlikely(!Py_IsInitialized())) { // LCOV_BR_EXCL_LINE
+        // LCOV_EXCL_START
         [super dealloc];
         return;
+        // LCOV_EXCL_STOP
     }
 
     PyObjC_BEGIN_WITH_GIL
@@ -101,9 +94,10 @@ extern NSString* const NSUnknownKeyException; /* Radar #3336042 */
     NSObject* result;
     PyObject* copy;
 
-    if (PyObjC_CopyFunc == NULL) {
-        [NSException raise:NSInvalidArgumentException
-                    format:@"cannot copy Python objects"];
+    if (PyObjC_CopyFunc == NULL || PyObjC_CopyFunc == Py_None) {
+        @throw [NSException exceptionWithName:NSInvalidArgumentException
+                                       reason:@"cannot copy Python objects"
+                                     userInfo:nil];
 
     } else {
         PyObjC_BEGIN_WITH_GIL
@@ -142,8 +136,8 @@ extern NSString* const NSUnknownKeyException; /* Radar #3336042 */
 {
     PyObject* repr;
 
-    if (pyObject == NULL)
-        return @"no python object";
+    if (pyObject == NULL)           // LCOV_BR_EXCL_LINE
+        return @"no python object"; // LCOV_EXCL_LINE
 
     PyObjC_BEGIN_WITH_GIL
 
@@ -153,48 +147,47 @@ extern NSString* const NSUnknownKeyException; /* Radar #3336042 */
             int       err;
             NSString* result;
 
-            /* XXX: use other function */
-            err = depythonify_c_value(@encode(id), repr, &result);
+            err = depythonify_python_object(repr, &result);
             Py_DECREF(repr);
             if (err == -1) {
                 PyObjC_GIL_FORWARD_EXC();
             }
 
             PyObjC_GIL_RETURN(result);
-        } else {
-            PyObjC_GIL_FORWARD_EXC();
         }
+        PyObjC_GIL_FORWARD_EXC();
 
     PyObjC_END_WITH_GIL
-
-    /* not reached */
-    return @"a python object";
 }
 
 - (void)doesNotRecognizeSelector:(SEL)aSelector
 {
-    [NSException raise:NSInvalidArgumentException
-                format:@"%@ does not recognize -%s", self, sel_getName(aSelector)];
+    @throw [NSException
+        exceptionWithName:NSInvalidArgumentException
+                   reason:[NSString stringWithFormat:@"%@ does not recognize -%s", self,
+                                                     sel_getName(aSelector)]
+                 userInfo:nil];
 }
 
-static inline PyObject* _Nullable check_argcount(PyObject* pymethod, Py_ssize_t argcount)
+static inline int
+check_argcount(PyObject* pymethod, Py_ssize_t argcount)
 {
     PyCodeObject* func_code;
 
     if (PyFunction_Check(pymethod)) {
         func_code = (PyCodeObject*)PyFunction_GetCode(pymethod);
         if (argcount == func_code->co_argcount) {
-            return pymethod;
+            return 0;
         }
 
     } else if (PyMethod_Check(pymethod)) {
         func_code = (PyCodeObject*)PyFunction_GetCode(PyMethod_Function(pymethod));
         if (argcount == func_code->co_argcount - 1) {
-            return pymethod;
+            return 0;
         }
     }
 
-    return NULL;
+    return -1;
 }
 
 /*#F If the Python object @var{obj} implements a method whose name matches
@@ -207,10 +200,13 @@ static PyObject* _Nullable get_method_for_selector(PyObject* obj, SEL aSelector)
     Py_ssize_t  argcount;
     PyObject*   pymethod;
     const char* p;
-    PyObject*   result;
 
-    if (!aSelector) {
-        [NSException raise:NSInvalidArgumentException format:@"nil selector"];
+    if (!aSelector) { // LCOV_BR_EXCL_LINE
+        // LCOV_EXCL_START
+        @throw [NSException exceptionWithName:NSInvalidArgumentException
+                                       reason:@"nil selector"
+                                     userInfo:nil];
+        // LCOV_EXCL_STOP
     }
 
     meth_name = sel_getName(aSelector);
@@ -222,7 +218,7 @@ static PyObject* _Nullable get_method_for_selector(PyObject* obj, SEL aSelector)
     }
 
     const char* py_meth_name =
-        PyObjC_SELToPythonName(aSelector, pymeth_name, sizeof(pymeth_name));
+        PyObjC_SELToPythonName(aSelector, pymeth_name, sizeof(pymeth_name) - 1);
     if (py_meth_name == NULL) {
         return NULL;
     }
@@ -230,41 +226,28 @@ static PyObject* _Nullable get_method_for_selector(PyObject* obj, SEL aSelector)
     if (pymethod == NULL) {
         return NULL;
     }
-
-    result = check_argcount(pymethod, argcount);
-    if (result == NULL) {
+    if (check_argcount(pymethod, argcount) == -1) {
         Py_DECREF(pymethod);
+        return NULL;
     }
-    return result;
+    return pymethod;
 }
 
 - (BOOL)respondsToSelector:(SEL)aSelector
 {
-    PyObject*    m;
-    Method*      methods;
-    unsigned int method_count;
-    unsigned int i;
-    void*        cookie;
+    PyObject* m;
+    Method    method;
 
     /*
      * We cannot rely on NSProxy, it doesn't implement most of the
      * NSObject interface anyway.
      */
 
-    cookie  = NULL;
-    methods = class_copyMethodList(object_getClass(self), &method_count);
-    if (methods == NULL) {
-        return NO;
+    method = class_getInstanceMethod(object_getClass(self), aSelector);
+    if (method != NULL) {
+        return YES;
     }
 
-    for (i = 0; i < method_count; i++) {
-        if (sel_isEqual(method_getName(methods[i]), aSelector)) {
-            free(methods);
-            return YES;
-        }
-    }
-    free(methods);
-
     PyObjC_BEGIN_WITH_GIL
         m = get_method_for_selector(pyObject, aSelector);
 
@@ -287,15 +270,11 @@ static PyObject* _Nullable get_method_for_selector(PyObject* obj, SEL aSelector)
     if (m) {
         /* A real Objective-C method */
         const char* typestr = method_getTypeEncoding(m);
-        if (typestr == NULL) {
-            return nil;
+        if (typestr == NULL) { // LCOV_BR_EXCL_LINE
+            return nil;        // LCOV_EXCL_LINE
         }
         return [NSMethodSignature signatureWithObjCTypes:typestr];
     }
-
-    [NSException
-         raise:NSInvalidArgumentException
-        format:@"Class %s: no such selector: %s", class_getName(self), sel_getName(sel)];
     return nil;
 }
 
@@ -314,11 +293,11 @@ static PyObject* _Nullable get_method_for_selector(PyObject* obj, SEL aSelector)
 
     cls = object_getClass(self);
     m   = class_getInstanceMethod(cls, sel);
-    if (m) {
+    if (m != NULL) {
         /* A real Objective-C method */
         const char* typestr = method_getTypeEncoding(m);
-        if (typestr == NULL) {
-            return nil;
+        if (typestr == NULL) { // LCOv_BR_EXCL_LINE
+            return nil;        // LCOV_EXCL_LINE
         }
         return [NSMethodSignature signatureWithObjCTypes:typestr];
     }
@@ -328,11 +307,7 @@ static PyObject* _Nullable get_method_for_selector(PyObject* obj, SEL aSelector)
         pymethod = get_method_for_selector(pyObject, sel);
         if (!pymethod) {
             PyErr_Clear();
-            PyObjC_LEAVE_GIL;
-            /* XXX: Use @throw */
-            [NSException raise:NSInvalidArgumentException
-                        format:@"Class %s: no such selector: %s",
-                               object_getClassName(self), sel_getName(sel)];
+            PyObjC_GIL_RETURN(nil);
         }
 
         if (PyMethod_Check(pymethod)) {
@@ -362,7 +337,6 @@ static PyObject* _Nullable get_method_for_selector(PyObject* obj, SEL aSelector)
     if (sel_isEqual(aSelector, @selector(description))) {
         id res = [self description];
         [invocation setReturnValue:&res];
-
         return YES;
 
     } else if (sel_isEqual(aSelector, @selector(_copyDescription))) {
@@ -481,8 +455,32 @@ static PyObject* _Nullable get_method_for_selector(PyObject* obj, SEL aSelector)
         [invocation setReturnValue:&c];
 
         return YES;
-    }
 
+    } else if (sel_isEqual(aSelector, @selector(doesNotRecognizeSelector:))) {
+        SEL sel;
+
+        [invocation getArgument:&sel atIndex:2];
+        [self doesNotRecognizeSelector:sel];
+
+        return YES;
+
+    } else if (sel_isEqual(aSelector, @selector(hash))) {
+        NSUInteger hash;
+
+        hash = [self hash];
+        [invocation setReturnValue:&hash];
+
+        return YES;
+    } else if (sel_isEqual(aSelector, @selector(methodSignatureForSelector:))) {
+        SEL       sel;
+        NSObject* result;
+
+        [invocation getArgument:&sel atIndex:2];
+        result = [self methodSignatureForSelector:sel];
+        [invocation setReturnValue:&result];
+
+        return YES;
+    }
     return NO;
 }
 
@@ -526,11 +524,22 @@ static PyObject* _Nullable get_method_for_selector(PyObject* obj, SEL aSelector)
             return;
         }
 
-        argcount = [msign numberOfArguments];
-        args     = PyTuple_New(argcount - 2);
-        if (args == NULL) {
+        @try {
+            argcount = [msign numberOfArguments];
+
+            // LCOV_EXCL_START
+        } @catch (NSObject* exc) {
+            PyObjC_LEAVE_GIL;
+            @throw;
+        }
+        // LCOV_EXCL_STOP
+
+        args = PyTuple_New(argcount - 2);
+        if (args == NULL) { // LCOV_BR_EXCL_LINE
+            // LCOV_EXCL_START
             Py_DECREF(pymethod);
             PyObjC_GIL_FORWARD_EXC();
+            // LCOV_EXCL_STOP
         }
         for (i = 2; i < argcount; i++) {
             const char* argtype;
@@ -540,10 +549,12 @@ static PyObject* _Nullable get_method_for_selector(PyObject* obj, SEL aSelector)
 
             @try {
                 argtype = [msign getArgumentTypeAtIndex:i];
+                // LCOV_EXCL_START
             } @catch (NSObject* exc) {
                 PyObjC_LEAVE_GIL;
                 @throw;
             }
+            // LCOV_EXCL_STOP
 
             argsize = PyObjCRT_SizeOfType(argtype);
             if (argsize == -1) {
@@ -557,10 +568,12 @@ static PyObject* _Nullable get_method_for_selector(PyObject* obj, SEL aSelector)
                 @try {
                     [invocation getArgument:argbuffer atIndex:i];
 
+                    // LCOV_EXCL_START
                 } @catch (NSObject* exc) {
                     PyObjC_LEAVE_GIL;
                     @throw;
                 }
+            // LCOV_EXCL_STOP
             Py_END_ALLOW_THREADS
 
             pyarg = pythonify_c_value(argtype, argbuffer);
@@ -592,22 +605,18 @@ static PyObject* _Nullable get_method_for_selector(PyObject* obj, SEL aSelector)
                 @try {
                     [invocation setReturnValue:retbuffer];
 
+                    // LCOV_EXCL_START
                 } @catch (NSObject* localException) {
                     PyObjC_LEAVE_GIL;
                     @throw;
                 }
+            // LCOV_EXCL_STOP
             Py_END_ALLOW_THREADS
         }
 
     PyObjC_END_WITH_GIL
 }
 
-/* XXX: Remove this method */
-- (PyObject*)pyObject
-{
-    return pyObject;
-}
-
 - (PyObject* _Nullable)__pyobjc_PythonObject__
 {
     PyObjC_BEGIN_WITH_GIL
@@ -662,58 +671,19 @@ static PyObject* _Nullable get_method_for_selector(PyObject* obj, SEL aSelector)
     return YES;
 }
 
-/* XXX:
- * - Try to use PyImport_ImportModuleLevel here
- * - Preferably switch to an objc.option for setting
- *   the function.
- */
-static PyObject* _Nullable getModuleFunction(char* modname, char* funcname)
-{
-    PyObject* func;
-    PyObject* name;
-    PyObject* mod;
-
-    name = PyUnicode_FromString(modname);
-    if (name == NULL) {
-        return NULL;
-    }
-
-    mod = PyImport_Import(name);
-    if (mod == NULL) {
-        Py_DECREF(name);
-        return NULL;
-    }
-    func = PyObject_GetAttrString(mod, funcname);
-    if (func == NULL) {
-        Py_DECREF(name);
-        Py_DECREF(mod);
-        return NULL;
-    }
-    Py_DECREF(name);
-    Py_DECREF(mod);
-
-    return func;
-}
-
-/*
- *  Call PyObjCTools.KeyValueCoding.getKey to get the value for a key
- */
 - (id)valueForKey:(NSString*)key
 {
-    static PyObject* getKeyFunc = NULL;
-
     PyObject* keyName;
     PyObject* val;
     id        res = nil;
 
-    PyObjC_BEGIN_WITH_GIL
+    if (PyObjC_getKey == NULL || PyObjC_getKey == Py_None) {
+        @throw [NSException exceptionWithName:NSInvalidArgumentException
+                                       reason:@"helper function for getKey not set"
+                                     userInfo:nil];
+    }
 
-        if (getKeyFunc == NULL) {
-            getKeyFunc = getModuleFunction("PyObjCTools.KeyValueCoding", "getKey");
-            if (getKeyFunc == NULL) {
-                PyObjC_GIL_FORWARD_EXC();
-            }
-        }
+    PyObjC_BEGIN_WITH_GIL
 
         keyName = id_to_python(key);
         if (keyName == NULL) {
@@ -722,7 +692,7 @@ static PyObject* _Nullable getModuleFunction(char* modname, char* funcname)
 
         PyObject* args[3] = {NULL, pyObject, keyName};
 
-        val = PyObject_Vectorcall(getKeyFunc, args + 1,
+        val = PyObject_Vectorcall(PyObjC_getKey, args + 1,
                                   2 | PY_VECTORCALL_ARGUMENTS_OFFSET, NULL);
         Py_DECREF(keyName);
         if (val == NULL) {
@@ -752,20 +722,17 @@ static PyObject* _Nullable getModuleFunction(char* modname, char* funcname)
 
 - (void)setValue:value forKey:(NSString*)key
 {
-    static PyObject* setKeyFunc = NULL;
-
     PyObject* keyName;
     PyObject* pyValue;
     PyObject* val;
 
-    PyObjC_BEGIN_WITH_GIL
+    if (PyObjC_setKey == NULL || PyObjC_setKey == Py_None) {
+        @throw [NSException exceptionWithName:NSInvalidArgumentException
+                                       reason:@"helper function for setKey not set"
+                                     userInfo:nil];
+    }
 
-        if (setKeyFunc == NULL) {
-            setKeyFunc = getModuleFunction("PyObjCTools.KeyValueCoding", "setKey");
-            if (setKeyFunc == NULL) {
-                PyObjC_GIL_FORWARD_EXC();
-            }
-        }
+    PyObjC_BEGIN_WITH_GIL
 
         keyName = id_to_python(key);
         if (keyName == NULL) {
@@ -780,7 +747,7 @@ static PyObject* _Nullable getModuleFunction(char* modname, char* funcname)
 
         PyObject* args[4] = {NULL, pyObject, keyName, pyValue};
 
-        val = PyObject_Vectorcall(setKeyFunc, args + 1,
+        val = PyObject_Vectorcall(PyObjC_setKey, args + 1,
                                   3 | PY_VECTORCALL_ARGUMENTS_OFFSET, NULL);
         Py_DECREF(keyName);
         Py_DECREF(pyValue);
@@ -817,21 +784,17 @@ static PyObject* _Nullable getModuleFunction(char* modname, char* funcname)
 
 - (id _Nullable)valueForKeyPath:(NSString*)keyPath
 {
-    static PyObject* getKeyFunc = NULL;
-
     PyObject* keyName;
     PyObject* val;
     id        res = nil;
 
+    if (PyObjC_getKeyPath == NULL || PyObjC_getKeyPath == Py_None) {
+        @throw [NSException exceptionWithName:NSInvalidArgumentException
+                                       reason:@"helper function for getKeyPath not set"
+                                     userInfo:nil];
+    }
+
     PyObjC_BEGIN_WITH_GIL
-
-        if (getKeyFunc == NULL) {
-            getKeyFunc = getModuleFunction("PyObjCTools.KeyValueCoding", "getKeyPath");
-            if (getKeyFunc == NULL) {
-                PyObjC_GIL_FORWARD_EXC();
-            }
-        }
-
         keyName = id_to_python(keyPath);
         if (keyName == NULL) {
             PyObjC_GIL_FORWARD_EXC();
@@ -839,7 +802,7 @@ static PyObject* _Nullable getModuleFunction(char* modname, char* funcname)
 
         PyObject* args[3] = {NULL, pyObject, keyName};
 
-        val = PyObject_Vectorcall(getKeyFunc, args + 1,
+        val = PyObject_Vectorcall(PyObjC_getKeyPath, args + 1,
                                   2 | PY_VECTORCALL_ARGUMENTS_OFFSET, NULL);
         Py_DECREF(keyName);
         if (val == NULL) {
@@ -864,21 +827,17 @@ static PyObject* _Nullable getModuleFunction(char* modname, char* funcname)
 
 - (void)setValue:value forKeyPath:(NSString*)keyPath
 {
-    static PyObject* setKeyFunc = NULL;
-
     PyObject* keyName;
     PyObject* pyValue;
     PyObject* val;
 
+    if (PyObjC_setKeyPath == NULL || PyObjC_setKeyPath == Py_None) {
+        @throw [NSException exceptionWithName:NSInvalidArgumentException
+                                       reason:@"helper function for setKeyPath not set"
+                                     userInfo:nil];
+    }
+
     PyObjC_BEGIN_WITH_GIL
-
-        if (setKeyFunc == NULL) {
-            setKeyFunc = getModuleFunction("PyObjCTools.KeyValueCoding", "setKeyPath");
-            if (setKeyFunc == NULL) {
-                PyObjC_GIL_FORWARD_EXC();
-            }
-        }
-
         keyName = id_to_python(keyPath);
         if (keyName == NULL) {
             PyObjC_GIL_FORWARD_EXC();
@@ -892,7 +851,7 @@ static PyObject* _Nullable getModuleFunction(char* modname, char* funcname)
 
         PyObject* args[4] = {NULL, pyObject, keyName, pyValue};
 
-        val = PyObject_Vectorcall(setKeyFunc, args + 1,
+        val = PyObject_Vectorcall(PyObjC_setKeyPath, args + 1,
                                   3 | PY_VECTORCALL_ARGUMENTS_OFFSET, NULL);
         Py_DECREF(keyName);
         Py_DECREF(pyValue);
@@ -924,17 +883,23 @@ static PyObject* _Nullable getModuleFunction(char* modname, char* funcname)
 
 - (void)unableToSetNilForKey:(NSString*)key
 {
-    [NSException raise:NSUnknownKeyException format:@"cannot set Nil for key: %@", key];
+    @throw [NSException
+        exceptionWithName:NSUndefinedKeyException
+                   reason:[NSString stringWithFormat:@"cannot set Nil for key: %@", key]
+                 userInfo:nil];
 }
 
-- (void)handleQueryWithUnboundKey:(NSString*)key
+- (id)handleQueryWithUnboundKey:(NSString*)key
 {
-    [self valueForUndefinedKey:key];
+    return [self valueForUndefinedKey:key];
 }
 
-- (void)valueForUndefinedKey:(NSString*)key
+- (id)valueForUndefinedKey:(NSString*)key
 {
-    [NSException raise:NSUnknownKeyException format:@"query for unknown key: %@", key];
+    @throw [NSException
+        exceptionWithName:NSUndefinedKeyException
+                   reason:[NSString stringWithFormat:@"query for unknown key: %@", key]
+                 userInfo:nil];
 }
 
 - (void)handleTakeValue:value forUnboundKey:(NSString*)key
@@ -944,11 +909,13 @@ static PyObject* _Nullable getModuleFunction(char* modname, char* funcname)
 
 - (void)setValue:value forUndefinedKey:(NSString*)key
 {
-    [NSException raise:NSUnknownKeyException
-                format:@"setting unknown key: %@ to <%@>", key, value];
+    @throw [NSException
+        exceptionWithName:NSUndefinedKeyException
+                   reason:[NSString stringWithFormat:@"setting unknown key: %@ to <%@>",
+                                                     key, value]
+                 userInfo:nil];
 }
 
-#if MAC_OS_X_VERSION_MAX_ALLOWED >= MAC_OS_X_VERSION_10_3
 - (void)addObserver:(NSObject*)observer
          forKeyPath:(NSString*)keyPath
             options:(NSKeyValueObservingOptions)options
@@ -963,7 +930,6 @@ static PyObject* _Nullable getModuleFunction(char* modname, char* funcname)
     NSLog(@"*** Ignoring *** %@ for '%@' (of %@).", NSStringFromSelector(_cmd), keyPath,
           observer);
 }
-#endif
 
 /* NSObject protocol */
 - (NSUInteger)hash
@@ -971,10 +937,10 @@ static PyObject* _Nullable getModuleFunction(char* modname, char* funcname)
     Py_hash_t rval;
 
     PyObjC_BEGIN_WITH_GIL
-        rval = PyObject_Hash([self pyObject]);
+        rval = PyObject_Hash(pyObject);
         if (rval == -1) {
             PyErr_Clear();
-            rval = (NSUInteger)[self pyObject];
+            rval = (NSUInteger)pyObject;
         }
 
     PyObjC_END_WITH_GIL
@@ -997,10 +963,10 @@ static PyObject* _Nullable getModuleFunction(char* modname, char* funcname)
             PyErr_Clear();
             PyObjC_GIL_RETURN(NO);
         }
-        if (otherPyObject == [self pyObject]) {
+        if (otherPyObject == pyObject) {
             PyObjC_GIL_RETURN(YES);
         }
-        switch (PyObject_RichCompareBool([self pyObject], otherPyObject, Py_EQ)) {
+        switch (PyObject_RichCompareBool(pyObject, otherPyObject, Py_EQ)) {
         case -1:
             PyErr_Clear();
         case 0:
@@ -1016,7 +982,9 @@ static PyObject* _Nullable getModuleFunction(char* modname, char* funcname)
 - (NSComparisonResult)compare:(id)other
 {
     if (other == nil) {
-        [NSException raise:NSInvalidArgumentException format:@"nil argument"];
+        @throw [NSException exceptionWithName:NSInvalidArgumentException
+                                       reason:@"nil argument"
+                                     userInfo:nil];
     } else if (self == other) {
         return NSOrderedSame;
     }
@@ -1025,11 +993,11 @@ static PyObject* _Nullable getModuleFunction(char* modname, char* funcname)
         if (otherPyObject == NULL) {
             PyObjC_GIL_FORWARD_EXC();
         }
-        if (otherPyObject == [self pyObject]) {
+        if (otherPyObject == pyObject) {
             PyObjC_GIL_RETURN(NSOrderedSame);
         }
         int r;
-        if (PyObjC_Cmp([self pyObject], otherPyObject, &r) == -1) {
+        if (PyObjC_Cmp(pyObject, otherPyObject, &r) == -1) {
             PyObjC_GIL_FORWARD_EXC();
         }
         NSComparisonResult rval;
@@ -1072,7 +1040,7 @@ static PyObject* _Nullable getModuleFunction(char* modname, char* funcname)
 {
     pyObject = NULL;
 
-    if (PyObjC_Decoder != NULL) {
+    if (PyObjC_Decoder != NULL && PyObjC_Decoder != Py_None) {
         PyObjC_BEGIN_WITH_GIL
             PyObject* cdr = id_to_python(coder);
             PyObject* setValue;
@@ -1084,10 +1052,16 @@ static PyObject* _Nullable getModuleFunction(char* modname, char* funcname)
             }
 
             selfAsPython = PyObjCObject_New(self, 0, YES);
-            if (selfAsPython == NULL) {
-                PyObjC_GIL_FORWARD_EXC();
+            if (selfAsPython == NULL) {   // LCOV_BR_EXCL_LINE
+                PyObjC_GIL_FORWARD_EXC(); // LCOV_EXCL_LINE
             }
             setValue = PyObject_GetAttrString(selfAsPython, "pyobjcSetValue_");
+            if (setValue == NULL) { // LCOV_BR_EXCL_LINE
+                // LCOV_EXCL_STOP
+                Py_DECREF(selfAsPython);
+                PyObjC_GIL_FORWARD_EXC();
+                // LCOV_EXCL_STOP
+            }
 
             v = PyObjC_CallDecoder(cdr, setValue);
             Py_DECREF(cdr);
@@ -1121,9 +1095,9 @@ static PyObject* _Nullable getModuleFunction(char* modname, char* funcname)
         return self;
 
     } else {
-        [NSException raise:NSInvalidArgumentException
-                    format:@"decoding Python objects is not supported"];
-        return nil;
+        @throw [NSException exceptionWithName:NSInvalidArgumentException
+                                       reason:@"decoding Python objects is not supported"
+                                     userInfo:nil];
     }
 }
 
@@ -1169,11 +1143,6 @@ static PyObject* _Nullable getModuleFunction(char* modname, char* funcname)
     return [OC_PythonObject class];
 }
 
-+ (Class)classForUnarchiver
-{
-    return [OC_PythonObject class];
-}
-
 + (Class)classForKeyedUnarchiver
 {
     return [OC_PythonObject class];
@@ -1268,11 +1237,14 @@ static CFTypeID _NSObjectTypeID;
 void
 PyObjC_encodeWithCoder(PyObject* pyObject, NSCoder* coder)
 {
-    if (PyObjC_Encoder != NULL) {
+    /* XXX: This should be called with the GIL held, and should
+     * return an error indicator.
+     */
+    if (PyObjC_Encoder != NULL && PyObjC_Encoder != Py_None) {
         PyObjC_BEGIN_WITH_GIL
             PyObject* cdr = id_to_python(coder);
-            if (cdr == NULL) {
-                PyObjC_GIL_FORWARD_EXC();
+            if (cdr == NULL) {            // LCOV_BR_EXCL_LINE
+                PyObjC_GIL_FORWARD_EXC(); // LCOV_EXCL_LINE
             }
 
             PyObject* args[3] = {NULL, pyObject, cdr};
@@ -1280,18 +1252,17 @@ PyObjC_encodeWithCoder(PyObject* pyObject, NSCoder* coder)
             PyObject* r = PyObject_Vectorcall(PyObjC_Encoder, args + 1,
                                               2 | PY_VECTORCALL_ARGUMENTS_OFFSET, NULL);
             Py_DECREF(cdr);
+            Py_XDECREF(r);
             if (r == NULL) {
                 PyObjC_GIL_FORWARD_EXC();
-
-            } else {
-                Py_DECREF(r);
             }
 
         PyObjC_END_WITH_GIL
 
     } else {
-        [NSException raise:NSInvalidArgumentException
-                    format:@"encoding Python objects is not supported"];
+        @throw [NSException exceptionWithName:NSInvalidArgumentException
+                                       reason:@"encoding Python objects is not supported"
+                                     userInfo:nil];
     }
 }
 
