@@ -12,8 +12,8 @@ NS_ASSUME_NONNULL_BEGIN
 - (instancetype _Nullable)initWithPythonObject:(PyObject*)v
 {
     self = [super init];
-    if (unlikely(self == nil))
-        return nil;
+    if (unlikely(self == nil)) // LCOV_BR_EXCL_LINE
+        return nil;            // LCOV_EXCL_LINE
 
     SET_FIELD_INCREF(value, v);
     return self;
@@ -59,9 +59,11 @@ NS_ASSUME_NONNULL_BEGIN
      * the call to Py_Finalize (shutting down the interpreter) and the
      * cleanup performed by Cocoa, possible on other threads.
      */
-    if (unlikely(!Py_IsInitialized())) {
+    if (unlikely(!Py_IsInitialized())) { // LCOV_BR_EXCL_LINE
+        // LCOV_EXCL_START
         [super release];
         return;
+        // LCOV_EXCL_STOP
     }
 
     PyObjC_BEGIN_WITH_GIL
@@ -77,27 +79,25 @@ NS_ASSUME_NONNULL_BEGIN
 
 - (void)dealloc
 {
-    if (unlikely(!Py_IsInitialized())) {
+    if (unlikely(!Py_IsInitialized())) { // LCOV_BR_EXCL_LINE
+        // LCOV_EXCL_START
         [super dealloc];
         return;
+        // LCOV_EXCL_STOP
     }
     PyObjC_BEGIN_WITH_GIL
         PyObjC_UnregisterObjCProxy(value, self);
         @try {
             [realObject release];
-        } @catch (NSObject* exc) {
+        } @catch (NSObject* exc) { // LCOV_BR_EXCL_LINE
+            // LCOV_EXCL_START
             PyObjC_LEAVE_GIL;
             @throw;
+            // LCOV_EXCL_STOP
         }
         realObject = nil;
         Py_CLEAR(value);
 
-#ifdef PyObjC_STR_CACHE_IMP
-        imp_length        = 0xDEADBEEF;
-        imp_charAtIndex   = 0xDEADBEEF;
-        imp_getCharacters = 0xDEADBEEF;
-#endif /* PyObjC_STR_CACHE_IMP */
-
     PyObjC_END_WITH_GIL
 
     [super dealloc];
@@ -106,13 +106,15 @@ NS_ASSUME_NONNULL_BEGIN
 - (id _Nullable)__realObject__
 {
 #ifdef Py_DEBUG
-    if (!PyUnicode_IS_READY(value)) {
+    if (!PyUnicode_IS_READY(value)) { // LCOV_BR_EXCL_LINE
+        // LCOV_EXCL_START
         /* Object should be ready, ensure we crash with the GIL
          * held when it's not.
          */
         PyObjC_BEGIN_WITH_GIL
             PyUnicode_GET_LENGTH(value);
         PyObjC_END_WITH_GIL
+        // LCOV_EXCL_STOP
     }
 #endif
 
@@ -213,35 +215,23 @@ NS_ASSUME_NONNULL_BEGIN
                        length:(NSUInteger)length
                      encoding:(NSStringEncoding)encoding
 {
-    char* py_encoding = NULL;
-    int   byteorder   = 0;
+    int byteorder = 0;
 
     /*
      * Call the super initializer first.
      */
     self = [super init];
-    if (self == nil) {
-        return nil;
+    if (self == nil) { // LCOV_BR_EXCL_LINE
+        return nil;    // LCOV_EXCL_LINE
     }
 
-    /* Detect some often used single-byte encodings that can be created in Python without
-     * creating an intermediate object.
+    /*
+     * The most common encoding is UTF-8, use a shortcut
+     * for that.
      */
-    switch (encoding) {
-    case NSASCIIStringEncoding:
-        py_encoding = "ascii";
-        break;
-    case NSUTF8StringEncoding:
-        py_encoding = "UTF-8";
-        break;
-    case NSISOLatin1StringEncoding:
-        py_encoding = "latin1";
-        break;
-    }
-
-    if (py_encoding != NULL) {
+    if (encoding == NSUTF8StringEncoding) {
         PyObjC_BEGIN_WITH_GIL
-            value = PyUnicode_Decode(bytes, length, py_encoding, NULL);
+            value = PyUnicode_DecodeUTF8(bytes, length, NULL);
             if (value == NULL) {
                 PyObjC_GIL_FORWARD_EXC();
             }
@@ -249,31 +239,6 @@ NS_ASSUME_NONNULL_BEGIN
         return self;
     }
 
-    /* UTF-16 encodings can also be decoded without an intermediate object */
-    byteorder = 2;
-    switch (encoding) {
-    case NSASCIIStringEncoding:
-        byteorder = 0;
-        break;
-    case NSUTF8StringEncoding:
-        byteorder = -1;
-        break;
-    case NSISOLatin1StringEncoding:
-        byteorder = 1;
-        break;
-    }
-    if (byteorder != 2) {
-        PyObjC_BEGIN_WITH_GIL
-            /* Decode as a UTF-16 string in native byteorder */
-            value = PyUnicode_DecodeUTF16(bytes, length, NULL, &byteorder);
-            if (value == NULL) {
-                PyObjC_GIL_FORWARD_EXC();
-            }
-
-        PyObjC_END_WITH_GIL;
-        return self;
-    }
-
     /* And finally: first use the Cocoa decoder to create an NSString, copy the unichars
      * into a temporary buffer and use that to create a Python unicode string using the
      * UTF16 decoder.
@@ -293,12 +258,14 @@ NS_ASSUME_NONNULL_BEGIN
      */
     unichar* chars = malloc(charcount * 2);
 
-    if (chars == NULL) {
+    if (chars == NULL) { // LCOV_BR_EXCL_LINE
+        // LCOV_EXCL_START
         [tmpval release];
         [self release];
         return nil;
+        // LCOV_EXCL_STOP
     }
-    [tmpval getCharacters:chars];
+    [tmpval getCharacters:chars range:NSMakeRange(0, charcount)];
     [tmpval release];
 
     PyObjC_BEGIN_WITH_GIL
@@ -306,8 +273,10 @@ NS_ASSUME_NONNULL_BEGIN
         byteorder = 0;
         value = PyUnicode_DecodeUTF16((const char*)chars, length * 2, NULL, &byteorder);
         free(chars);
-        if (value == NULL) {
+        if (value == NULL) { // LCOV_BR_EXCL_LINE
+            //  LCOV_EXCL_START
             PyObjC_GIL_FORWARD_EXC();
+            //  LCOV_EXCL_STOP
         }
 
     PyObjC_END_WITH_GIL;
@@ -343,20 +312,20 @@ NS_ASSUME_NONNULL_BEGIN
         return self;
     } else if (ver == 2) {
 
-        if (PyObjC_Decoder != NULL) {
+        if (PyObjC_Decoder != NULL && PyObjC_Decoder != Py_None) {
             PyObjC_BEGIN_WITH_GIL
-                PyObject* cdr = id_to_python(coder);
                 PyObject* setValue;
                 PyObject* selfAsPython;
                 PyObject* v;
 
-                if (cdr == NULL) {
-                    PyObjC_GIL_FORWARD_EXC();
+                PyObject* cdr = id_to_python(coder);
+                if (cdr == NULL) {            // LCOV_BR_EXCL_LINE
+                    PyObjC_GIL_FORWARD_EXC(); // LCOV_EXCL_LINE
                 }
 
                 selfAsPython = PyObjCObject_New(self, 0, YES);
-                if (selfAsPython == NULL) {
-                    PyObjC_GIL_FORWARD_EXC();
+                if (selfAsPython == NULL) {   // LCOV_BR_EXCL_LINE
+                    PyObjC_GIL_FORWARD_EXC(); // LCOV_EXCL_LINE
                 }
                 setValue = PyObject_GetAttrString(selfAsPython, "pyobjcSetValue_");
 
@@ -377,15 +346,19 @@ NS_ASSUME_NONNULL_BEGIN
 
             return self;
 
-        } else {
-            [NSException raise:NSInvalidArgumentException
-                        format:@"decoding Python objects is not supported"];
+        } else { // LCOV_BR_EXCL_LINE
+            // LOCV_EXCL_START
+            @throw
+                [NSException exceptionWithName:NSInvalidArgumentException
+                                        reason:@"decoding Python objects is not supported"
+                                      userInfo:nil];
             return nil;
+            // LOCV_EXCL_STOP
         }
     } else {
-        [NSException raise:NSInvalidArgumentException
-                    format:@"encoding Python unicode objects is not supported"];
-        return nil;
+        @throw [NSException exceptionWithName:NSInvalidArgumentException
+                                       reason:@"decoding Python objects is not supported"
+                                     userInfo:nil];
     }
 }
 
