@@ -440,17 +440,16 @@ PyDoc_STRVAR(loadBundle_doc,
 static PyObject* _Nullable loadBundle(PyObject* self __attribute__((__unused__)),
                                       PyObject* _Nullable args, PyObject* _Nullable kwds)
 {
-    static char*      keywords[] = {"module_name",       "module_globals", "bundle_path",
+    static char* keywords[] = {"module_name",       "module_globals", "bundle_path",
                                "bundle_identifier", "scan_classes",   NULL};
-    static Py_ssize_t curClassCount     = -1;
-    NSBundle*         bundle            = nil;
-    id                bundle_identifier = nil;
-    id                bundle_path       = nil;
-    PyObject*         module_name;
-    PyObject*         module_globals;
-    PyObject*         class_list;
-    Py_ssize_t        len, i;
-    PyObject*         scanClasses = NULL;
+    NSBundle*    bundle     = nil;
+    id           bundle_identifier = nil;
+    id           bundle_path       = nil;
+    PyObject*    module_name;
+    PyObject*    module_globals;
+    PyObject*    class_list;
+    Py_ssize_t   len, i;
+    PyObject*    scanClasses = NULL;
 
     if (!PyArg_ParseTupleAndKeywords(args, kwds, "UO|O&O&O", keywords, &module_name,
                                      &module_globals, PyObjCObject_Convert, &bundle_path,
@@ -503,7 +502,7 @@ static PyObject* _Nullable loadBundle(PyObject* self __attribute__((__unused__))
         return NULL;
     }
 
-    curClassCount = len = PyTuple_GET_SIZE(class_list);
+    len = PyTuple_GET_SIZE(class_list);
     for (i = 0; i < len; i++) {
         PyObject*   item;
         const char* nm;
@@ -867,7 +866,14 @@ static PyObject* _Nullable protocolsForClass(PyObject* self __attribute__((__unu
             return NULL;
             // LCOV_EXCL_STOP
         }
-        PyList_Append(protocols, protocol);
+        if (PyList_Append(protocols, protocol) == -1) { // LCOV_BR_EXCL_LINE
+            // LCOV_EXCL_START
+            Py_DECREF(protocol);
+            Py_DECREF(protocols);
+            free(protocol_list);
+            return NULL;
+            // LCOV_EXCL_STOP
+        }
         Py_DECREF(protocol);
     }
 
@@ -1967,6 +1973,8 @@ struct objc_typestr_long_values {
                                 {"_C_CFIndex", @encode(CFIndex)},
                                 {"_C_CGFloat", @encode(CGFloat)},
                                 {"_C_FSRef", @encode(FSRef)},
+                                {"_C_NSRange", @encode(NSRange)},
+                                {"_C_CFRange", @encode(CFRange)},
                                 {"_sockaddr_type", @encode(struct sockaddr)},
                                 {NULL, 0}};
 
@@ -2099,6 +2107,10 @@ PyObject* _Nullable __attribute__((__visibility__("default"))) PyInit__objc(void
         return NULL;                 // LCOV_EXCL_LINE
     }
 
+    if (PyObjC_SockAddr_Setup() == -1) { // LCOV_BR_EXCL_LINE
+        return NULL;                     // LCOV_EXCL_LINE
+    }
+
     /* XXX:
      *  Come up with an abstraction that readies a list of types and adds
      *  tem to the module dict
