@@ -37,21 +37,19 @@ struct PyObjC_method {
 #define _C_CHAR_AS_INT 'z'
 #define _C_NSBOOL 'Z'
 
-#define objc_superSetReceiver(super, val) (super).receiver = (val)
-#define objc_superGetReceiver(super) ((super).receiver)
-
-#define objc_superSetClass(super, cls) (super).super_class = (cls)
-#define objc_superGetClass(super) ((super).super_class)
-
 /* Some functions that are missing (oddly enough) */
-BOOL PyObjC_class_isSubclassOf(Class child, Class _Nullable parent);
-#define class_isSubclassOf PyObjC_class_isSubclassOf
+BOOL PyObjC_class_isSubclassOf(Class child, Class parent);
 
 /*
  * XXX: Override protocol_getMethodDescription. This is a crude hack that's added because
  * protocol_getMethodDescription sometimes gives the wrong answer (test_protocols.py).
  * I haven't found the root cause for this yet, it may or may not be a problem with
  * PyObjC.
+ *
+ * To make it easier to trest try to write a standalone program demonstrating the problem.
+ *
+ * XXX: Problem might be gone in 12.1, need more tests.
+ *
  */
 extern struct objc_method_description
 PyObjC_protocol_getMethodDescription(Protocol* p, SEL aSel, BOOL isRequiredMethod,
