@@ -29,6 +29,10 @@
 
 #include <CoreFoundation/CFNumber.h>
 
+#ifndef __LP64__
+#error "Requires LP64"
+#endif
+
 NS_ASSUME_NONNULL_BEGIN
 
 /*
@@ -66,28 +70,18 @@ NSObject (PyObjCSupport)
 
     if (rval == NULL) {
         rval = (PyObject*)PyObjCObject_New(self, PyObjCObject_kDEFAULT, YES);
-        if (rval == NULL) {
-            return NULL;
+        if (rval == NULL) { // LCOV_BR_EXCL_LINE
+            return NULL;    // LCOV_EXCL_LINE
         }
     }
 
-    if (rval != NULL) {
-        PyObjC_RegisterPythonProxy(self, rval);
-    }
-
+    PyObjC_RegisterPythonProxy(self, rval);
     return rval;
 }
 
 + (PyObject* _Nullable)__pyobjc_PythonObject__
 {
-    PyObject* rval;
-
-    rval = NULL;
-    if (rval == NULL) {
-        rval = (PyObject*)PyObjCClass_New(self);
-    }
-
-    return rval;
+    return (PyObject*)PyObjCClass_New(self);
 }
 
 - (PyObject* _Nullable)__pyobjc_PythonTransient__:(int*)cookie
@@ -211,7 +205,13 @@ NSString (PyObjCSupport)
 
 - (PyObject* _Nullable)__pyobjc_PythonObject__
 {
-    PyObject* rval = (PyObject*)PyObjCUnicode_New(self);
+    PyObject* rval = PyObjC_FindPythonProxy(self);
+    if (rval == NULL) {
+        rval = (PyObject*)PyObjCUnicode_New(self);
+        if (rval != NULL) {
+            PyObjC_RegisterPythonProxy(self, rval);
+        }
+    }
     return rval;
 }
 
@@ -327,8 +327,10 @@ PyObjCRT_SkipTypeQualifiers(const char* type)
            || *type == 'O') {
         type++;
     }
-    while (*type && isdigit(*type))
-        type++;
+    while (*type && isdigit(*type)) { // LCOV_BR_EXCL_LINE
+        /* XXX: There are no digits in after type qualifiers!. */
+        type++; // LCOV_EXCL_LINE
+    }
     return type;
 }
 
@@ -338,9 +340,6 @@ const char* _Nullable PyObjCRT_SkipTypeSpec(const char* start_type)
     PyObjC_Assert(start_type != NULL, NULL);
 
     type = PyObjCRT_SkipTypeQualifiers(type);
-    if (type == NULL) {
-        return NULL;
-    }
 
     switch (*type) {
     case '"':
@@ -509,9 +508,6 @@ const char* _Nullable PyObjCRT_NextField(const char* start_type)
     const char* _Nullable type = start_type;
 
     type = PyObjCRT_SkipTypeQualifiers(type);
-    if (type == NULL) {
-        return NULL;
-    }
 
     switch (*type) {
     /* The following are one character type codes */
@@ -719,8 +715,10 @@ PyObjCRT_AlignOfType(const char* start_type)
     case _C_UINT:
         return __alignof__(unsigned int);
     case _C_LNG:
+    case _C_LNG_LNG:
         return __alignof__(long);
     case _C_ULNG:
+    case _C_ULNG_LNG:
         return __alignof__(unsigned long);
     case _C_FLT:
         return __alignof__(float);
@@ -734,10 +732,6 @@ PyObjCRT_AlignOfType(const char* start_type)
 #endif
     case _C_PTR:
         return __alignof__(void*);
-    case _C_LNG_LNG:
-        return __alignof__(long long);
-    case _C_ULNG_LNG:
-        return __alignof__(unsigned long long);
     case _C_ARY_B:
         while (isdigit(*++type)) /* do nothing */
             ;
@@ -884,17 +878,15 @@ PyObjCRT_SizeOfType(const char* start_type)
     case _C_UINT:
         return sizeof(unsigned int);
     case _C_LNG:
+    case _C_LNG_LNG:
         return sizeof(long);
     case _C_ULNG:
+    case _C_ULNG_LNG:
         return sizeof(unsigned long);
     case _C_FLT:
         return sizeof(float);
     case _C_DBL:
         return sizeof(double);
-    case _C_LNG_LNG:
-        return sizeof(long long);
-    case _C_ULNG_LNG:
-        return sizeof(unsigned long long);
     case _C_UNICHAR:
         return sizeof(UniChar);
     case _C_CHAR_AS_TEXT:
@@ -940,7 +932,7 @@ PyObjCRT_SizeOfType(const char* start_type)
             == 0) {
 
             /* XXX: What about struct sockaddr_un? */
-            return sizeof(struct sockaddr_in6);
+            return sizeof(struct sockaddr_storage);
         }
 
         if (unlikely(IS_DECIMAL(type))) {
@@ -1054,6 +1046,7 @@ PyObject* _Nullable pythonify_c_array_nullterminated(const char* type, void* dat
     unsigned char* curdatum   = datum;
 
     type = PyObjCRT_SkipTypeQualifiers(type);
+
     switch (*type) {
     case _C_CHARPTR:
         while (*(char**)curdatum != NULL) {
@@ -1121,6 +1114,7 @@ PyObject* _Nullable pythonify_c_array_nullterminated(const char* type, void* dat
         break;
 
     case _C_ULNG:
+    case _C_ULNG_LNG:
         while (*(unsigned long*)curdatum != 0) {
             count++;
             curdatum += sizeofitem;
@@ -1128,21 +1122,8 @@ PyObject* _Nullable pythonify_c_array_nullterminated(const char* type, void* dat
         break;
 
     case _C_LNG:
-        while (*(long*)curdatum != 0) {
-            count++;
-            curdatum += sizeofitem;
-        }
-        break;
-
-    case _C_ULNG_LNG:
-        while (*(unsigned long long*)curdatum != 0) {
-            count++;
-            curdatum += sizeofitem;
-        }
-        break;
-
     case _C_LNG_LNG:
-        while (*(long long*)curdatum != 0) {
+        while (*(long*)curdatum != 0) {
             count++;
             curdatum += sizeofitem;
         }
@@ -1195,8 +1176,8 @@ static PyObject* _Nullable pythonify_c_array(const char* type, void* datum)
         return NULL;
 
     ret = PyTuple_New(nitems);
-    if (!ret)
-        return NULL;
+    if (!ret)        // LCOV_BR_EXCL_LINE
+        return NULL; // LCOV_EXCL_LINE
 
     curdatum = datum;
     for (itemidx = 0; itemidx < nitems; itemidx++) {
@@ -1293,8 +1274,8 @@ static PyObject* _Nullable pythonify_c_struct(const char* type, void* datum)
 
         haveTuple = 1;
         ret       = PyTuple_New(nitems);
-        if (!ret)
-            return NULL;
+        if (!ret)        // LCOV_BR_EXCL_LINE
+            return NULL; // LCOV_EXCL_LINE
 
         item = type;
 
@@ -1843,16 +1824,7 @@ pythonify_c_value(const char* type, void* datum)
         break;
 
     case _C_UINT:
-#if __LP64__
         retobject = (PyObject*)PyLong_FromLong(*(unsigned int*)datum);
-
-#else
-        if (*(unsigned int*)datum > LONG_MAX) {
-            retobject = (PyObject*)PyLong_FromUnsignedLongLong(*(unsigned int*)datum);
-        } else {
-            retobject = (PyObject*)PyLong_FromLong(*(unsigned int*)datum);
-        }
-#endif
         break;
 
     case _C_SHT:
@@ -1864,21 +1836,11 @@ pythonify_c_value(const char* type, void* datum)
         break;
 
     case _C_LNG_LNG:
-#ifndef __LP64__ /* else: fall-through to _C_LNG case */
-        retobject = (PyObject*)PyLong_FromLongLong(*(long long*)datum);
-        break;
-#endif
-
     case _C_LNG:
         retobject = (PyObject*)PyLong_FromLong(*(long*)datum);
         break;
 
     case _C_ULNG_LNG:
-#ifndef __LP64__ /* else: fallthrough to the ULNG case */
-        retobject = (PyObject*)PyLong_FromUnsignedLongLong(*(unsigned long long*)datum);
-        break;
-#endif
-
     case _C_ULNG:
         retobject = PyLong_FromUnsignedLong(*(unsigned long*)datum);
         break;
@@ -2241,7 +2203,7 @@ depythonify_python_object(PyObject* argument, id* datum)
     } else if (PyDict_Check(argument)) {
         *datum = [OC_PythonDictionary dictionaryWithPythonObject:argument];
 
-    } else if (PyBytes_CheckExact(argument)) {
+    } else if (PyBytes_CheckExact(argument) || PyByteArray_CheckExact(argument)) {
         *datum = [OC_BuiltinPythonData dataWithPythonObject:argument];
 
     } else if (PyObject_CheckBuffer(argument)) {
@@ -2307,6 +2269,26 @@ depythonify_python_object(PyObject* argument, id* datum)
             }
         }
 
+        if (*datum == nil && PyObjC_DateTime_Date_Type != NULL
+            && PyObjC_DateTime_Date_Type != Py_None) {
+            if ((PyObject*)Py_TYPE(argument) == PyObjC_DateTime_Date_Type) {
+                *datum = [OC_BuiltinPythonDate dateWithPythonObject:argument];
+                if (*datum == nil) {
+                    return -1;
+                }
+            }
+        }
+
+        if (*datum == nil && PyObjC_DateTime_DateTime_Type != NULL
+            && PyObjC_DateTime_DateTime_Type != Py_None) {
+            if ((PyObject*)Py_TYPE(argument) == PyObjC_DateTime_DateTime_Type) {
+                *datum = [OC_BuiltinPythonDate dateWithPythonObject:argument];
+                if (*datum == nil) {
+                    return -1;
+                }
+            }
+        }
+
         if (*datum == nil && PyObjC_DateLikeTypes != NULL) {
             int r;
 
@@ -2324,6 +2306,7 @@ depythonify_python_object(PyObject* argument, id* datum)
         }
 
         if (*datum == nil) {
+            PyObjC_Assert(!PyObjCObject_Check(argument), -1);
             *datum = [OC_PythonObject objectWithPythonObject:argument];
         }
     }
@@ -2797,9 +2780,6 @@ const char* _Nullable PyObjCRT_RemoveFieldNames(char* buf, const char* type)
     }
 
     end = PyObjCRT_SkipTypeQualifiers(type);
-    if (end == NULL) {
-        return NULL;
-    }
 
     switch (*end) {
     case _C_STRUCT_B:
