@@ -19,6 +19,8 @@ PyObject* PyObjCExc_ObjCPointerWarning;
 
 PyObject* PyObjCNM_insert;
 PyObject* PyObjCNM_append;
+PyObject* PyObjCNM_timestamp;
+PyObject* PyObjCNM_fromtimestamp;
 PyObject* PyObjCNM_strftime;
 PyObject* PyObjCNM_keys;
 PyObject* PyObjCNM_clear;
@@ -65,6 +67,8 @@ PyObjCUtil_Init(PyObject* module)
 
     NEW_STR(PyObjCNM_insert, "insert");
     NEW_STR(PyObjCNM_append, "append");
+    NEW_STR(PyObjCNM_timestamp, "timestamp");
+    NEW_STR(PyObjCNM_fromtimestamp, "fromtimestamp");
     NEW_STR(PyObjCNM_strftime, "strftime");
     NEW_STR(PyObjCNM_keys, "keys");
     NEW_STR(PyObjCNM_clear, "clear");
@@ -73,7 +77,7 @@ PyObjCUtil_Init(PyObject* module)
     NEW_STR(PyObjCNM_values, "values");
     NEW_STR(PyObjCNM_description, "description");
     NEW_STR(PyObjCNM___get__, "__get__");
-    NEW_STR(PyObjCNM_date_format_string, "%Y-%m-%d %H:%M:%S %z");
+    NEW_STR(PyObjCNM_date_format_string, "%s");
     NEW_STR(PyObjCNM_objc_memview_object, "objc.memview object");
     NEW_STR(PyObjCNM_objc_NULL, "objc.NULL");
 
@@ -132,7 +136,9 @@ PyObjCErr_FromObjC(NSObject* localException)
             if (exc == NULL) {
                 PyErr_Clear();
             } else {
-                PyObject_SetAttrString(exc_value, "_pyobjc_exc_", exc);
+                if (PyObject_SetAttrString(exc_value, "_pyobjc_exc_", exc) == -1) {
+                    PyErr_Clear();
+                }
             }
             Py_CLEAR(exc);
             PyErr_Restore(exc_type, exc_value, exc_traceback);
@@ -146,20 +152,17 @@ PyObjCErr_FromObjC(NSObject* localException)
 
                 val = [userInfo objectForKey:@"__pyobjc_exc_type__"];
                 if (val) {
-                    exc_type = [val pyObject];
-                    exc_value =
-                        [[userInfo objectForKey:@"__pyobjc_exc_value__"] pyObject];
-                    exc_traceback =
-                        [[userInfo objectForKey:@"__pyobjc_exc_traceback__"] pyObject];
+                    id temp;
+                    exc_type = id_to_python(val);
 
-                    /* -pyObject returns a borrowed reference and
-                     * PyErr_Restore steals one from us.
-                     */
-                    Py_INCREF(exc_type);
-                    Py_XINCREF(exc_value);
-                    Py_XINCREF(exc_traceback);
+                    temp          = [userInfo objectForKey:@"__pyobjc_exc_value__"];
+                    exc_value     = temp != NULL ? id_to_python(temp) : NULL;
+                    temp          = [userInfo objectForKey:@"__pyobjc_exc_traceback__"];
+                    exc_traceback = temp != NULL ? id_to_python(temp) : NULL;
 
-                    PyErr_Restore(exc_type, exc_value, exc_traceback);
+                    if (exc_type != NULL) {
+                        PyErr_Restore(exc_type, exc_value, exc_traceback);
+                    }
 
                     PyObjC_GIL_RETURNVOID;
                 }
@@ -186,21 +189,29 @@ PyObjCErr_FromObjC(NSObject* localException)
             /* Ignore errors in setting up ``dict``, the exception state
              * will be replaced later.
              */
-            PyDict_SetItemString(dict, "name", c_localException_name);
+            if (PyDict_SetItemString(dict, "name", c_localException_name) == -1) {
+                PyErr_Clear();
+            }
             Py_DECREF(c_localException_name);
 
-            PyDict_SetItemString(dict, "reason", c_localException_reason);
+            if (PyDict_SetItemString(dict, "reason", c_localException_reason) == -1) {
+                PyErr_Clear();
+            }
             Py_DECREF(c_localException_reason);
             if (userInfo) {
                 v = id_to_python(userInfo);
                 if (v != NULL) {
-                    PyDict_SetItemString(dict, "userInfo", v);
+                    if (PyDict_SetItemString(dict, "userInfo", v) == -1) {
+                        PyErr_Clear();
+                    }
                     Py_DECREF(v);
                 } else {           // LCOV_BR_EXCL_LINE
                     PyErr_Clear(); // LCOV_EXCL_LINE
                 }
             } else {
-                PyDict_SetItemString(dict, "userInfo", Py_None);
+                if (PyDict_SetItemString(dict, "userInfo", Py_None) == -1) {
+                    PyErr_Clear();
+                }
             }
 
             const char* name   = [[(NSException*)localException name] UTF8String];
@@ -213,9 +224,13 @@ PyObjCErr_FromObjC(NSObject* localException)
             PyErr_Fetch(&exc_type, &exc_value, &exc_traceback);
             PyErr_NormalizeException(&exc_type, &exc_value, &exc_traceback);
 
-            PyObject_SetAttrString(exc_value, "_pyobjc_info_", dict);
+            if (PyObject_SetAttrString(exc_value, "_pyobjc_info_", dict) == -1) {
+                PyErr_Clear();
+            }
             Py_CLEAR(dict);
-            PyObject_SetAttrString(exc_value, "name", c_localException_name);
+            if (PyObject_SetAttrString(exc_value, "name", c_localException_name) == -1) {
+                PyErr_Clear();
+            }
             PyErr_Restore(exc_type, exc_value, exc_traceback);
         }
     PyObjC_END_WITH_GIL
@@ -238,7 +253,6 @@ PyObjCErr_AsExc(void)
      */
     PyErr_Fetch(&exc_type, &exc_value, &exc_traceback);
     if (exc_type == NULL) {
-        printf("No exception!\n");
         return nil;
     }
 
@@ -308,6 +322,11 @@ PyObjCErr_AsExc(void)
     repr     = PyObject_Str(exc_value);
     typerepr = PyObject_Str(exc_type);
     userInfo = [NSMutableDictionary dictionaryWithCapacity:3];
+
+    /* XXX: For recent enough versions of Python we don't need to store all three */
+    // PyObjC_Assert(exc_type != NULL, nil);
+    // PyObjC_Assert(exc_value != NULL, nil);
+    // PyObjC_Assert(exc_traceback != NULL, nil);
     [userInfo setObject:[[[OC_PythonObject alloc] initWithPyObject:exc_type] autorelease]
                  forKey:@"__pyobjc_exc_type__"];
 
@@ -370,9 +389,9 @@ PyObjCErr_ToObjCWithGILState(PyGILState_STATE* _Nonnull state)
     if (state) {
         PyGILState_Release(*state);
     }
-    if (PyGILState_Check()) {
-        printf("Still own GIL!\n");
-    }
+    // if (PyGILState_Check()) {
+    //     printf("Still own GIL!\n");
+    // }
     @throw exc;
     __builtin_unreachable();
 }
@@ -1347,7 +1366,6 @@ PyObjCRT_SignaturesEqual(const char* sig1, const char* sig2)
 PyObject* _Nullable PyObjC_FindSELInDict(PyObject* clsdict, SEL selector)
 {
     PyObject*  values;
-    PyObject*  seq;
     Py_ssize_t i, len;
 
     values = PyDict_Values(clsdict);
@@ -1355,27 +1373,22 @@ PyObject* _Nullable PyObjC_FindSELInDict(PyObject* clsdict, SEL selector)
         return NULL;
     }
 
-    seq = PySequence_Fast(values, "PyDict_Values result not a sequence");
-    if (seq == NULL) {
-        return NULL;
-    }
+    PyObjC_Assert(PyList_Check(values), NULL);
 
-    len = PySequence_Fast_GET_SIZE(seq);
+    len = PyList_GET_SIZE(values);
     for (i = 0; i < len; i++) {
-        PyObject* v = PySequence_Fast_GET_ITEM(seq, i);
+        PyObject* v = PyList_GET_ITEM(values, i);
 
         if (!PyObjCSelector_Check(v))
             continue;
 
         if (PyObjCSelector_GetSelector(v) == selector) {
-            Py_DECREF(seq);
             Py_DECREF(values);
             Py_INCREF(v);
             return v;
         }
     }
 
-    Py_DECREF(seq);
     Py_DECREF(values);
     return NULL;
 }
