@@ -5,7 +5,7 @@
  * Central include file for PyObjC.
  */
 
-#define OBJC_VERSION "8.2"
+#define OBJC_VERSION "8.3"
 
 #define PY_SSIZE_T_CLEAN
 #include <Python.h>
@@ -18,6 +18,10 @@
 
 #include <objc/objc.h>
 
+#ifndef __LP64__
+#error "No in 64-bit mode"
+#endif
+
 #ifndef NS_ASSUME_NONNULL_BEGIN
 /*
  * Old compiler without nullability support
@@ -102,6 +106,7 @@
 
 #import "OC_BuiltinPythonArray.h"
 #import "OC_BuiltinPythonData.h"
+#import "OC_BuiltinPythonDate.h"
 #import "OC_BuiltinPythonDictionary.h"
 #import "OC_BuiltinPythonNumber.h"
 #import "OC_BuiltinPythonSet.h"
