@@ -31,6 +31,7 @@ from PyObjCTools.TestSupport import (
     expectedFailureIf,
     os_level_key,
     os_release,
+    pyobjc_options,
 )
 
 import copyreg
@@ -603,11 +604,11 @@ class TestKeyedArchiveSimple(TestCase):
         self.assertIsInstance(v, int)
         self.assertEqual(v, sys.maxsize * 4)
 
-        buf = self.archiverClass.archivedDataWithRootObject_(sys.maxsize ** 4)
+        buf = self.archiverClass.archivedDataWithRootObject_(sys.maxsize**4)
         self.assertIsInstance(buf, NSData)
         v = self.unarchiverClass.unarchiveObjectWithData_(buf)
         self.assertIsInstance(v, int)
-        self.assertEqual(v, sys.maxsize ** 4)
+        self.assertEqual(v, sys.maxsize**4)
 
     def testSimpleLists(self):
         o = []
@@ -1187,11 +1188,18 @@ class TestKeyedArchivePlainPython(TestCase, test.pickletester.AbstractPickleTest
 
         self.assertEqual(v, x)
 
-        for val in (int(0), int(1), int(sys.maxsize), int(sys.maxsize * 128)):
+        for val in (
+            int(0),
+            int(1),
+            int(sys.maxsize),
+            int(sys.maxsize * 128),
+            2**63 + 1,
+        ):
             for x in val, -val:
-                buf = self.dumps(x)
-                v = self.loads(buf)
-                self.assertEqual(v, x)
+                with self.subTest(x=x):
+                    buf = self.dumps(x)
+                    v = self.loads(buf)
+                    self.assertEqual(v, x)
 
     # Overriden tests for extension codes, the test code checks
     # the actual byte stream.
@@ -1773,3 +1781,20 @@ class TestSecureArchivingPython(TestCase):
             archive.encodeObject_forKey_(value, "value")
 
         archive.finishEncoding()
+
+    def test_archive_without_encoder(self):
+        with pyobjc_options(_nscoding_encoder=None):
+            with self.assertRaisesRegex(
+                ValueError,
+                "NSInvalidArgumentException - encoding Python objects is not supported",
+            ):
+                NSKeyedArchiver.archivedDataWithRootObject_(object())
+
+        buf = NSKeyedArchiver.archivedDataWithRootObject_(object())
+
+        with pyobjc_options(_nscoding_decoder=None):
+            with self.assertRaisesRegex(
+                ValueError,
+                "NSInvalidArgumentException - decoding Python objects is not supported",
+            ):
+                NSKeyedUnarchiver.unarchiveObjectWithData_(buf)
