@@ -24,6 +24,36 @@ MyProto3 = objc.informal_protocol(
 
 
 class TestInformalProtocols(TestCase):
+    def test_repr(self):
+        v = repr(MyProto)
+        self.assertRegex(v, "<objc.informal_protocol 'MyProto' at .*>")
+
+    def test_creation_errors(self):
+        with self.assertRaisesRegex(
+            TypeError, r"informal_protocol\(\) argument 1 must be str, not int"
+        ):
+            objc.informal_protocol(42, ())
+
+        with self.assertRaisesRegex(TypeError, r"'int' object is not iterable"):
+            objc.informal_protocol("name", 42)
+
+        with self.assertRaisesRegex(TypeError, r"Item 0 is not a selector"):
+            objc.informal_protocol("name", (42,))
+
+        with self.assertRaisesRegex(TypeError, r"Item 1 is not a selector"):
+            objc.informal_protocol(
+                "name",
+                (
+                    objc.selector(
+                        None,
+                        selector=b"incompleteMethod",
+                        signature=b"I@:",
+                        isRequired=1,
+                    ),
+                    42,
+                ),
+            )
+
     def testMissingProto(self):
         class ProtoClass1(NSObject):
             def testMethod(self):
@@ -34,11 +64,11 @@ class TestInformalProtocols(TestCase):
     def testIncompleteClass(self):
         with self.assertRaisesRegex(
             TypeError,
-            r"metaclass conflict: the metaclass of a derived class must be "
-            r"a \(non-strict\) subclass of the metaclasses of all its bases",
+            "class ProtoClass2 does not fully implement protocol MyProto: "
+            "no implementation for testMethod",
         ):
 
-            class ProtoClass2(NSObject, MyProto):
+            class ProtoClass2(NSObject, protocols=[MyProto]):
                 def testMethod2_(self, x):
                     pass
 
@@ -48,6 +78,53 @@ class TestInformalProtocols(TestCase):
         for cls in objc.getClassList():
             self.assertNotEqual(cls.__name__, "ProtoClass2")
 
+    def testInvalidMethodType(self):
+        with self.assertRaisesRegex(
+            TypeError,
+            "class ProtoClass2 does not correctly implement protocol MyProto: "
+            "the signature for method testMethod is @@: instead of I@:",
+        ):
+
+            class ProtoClass2(NSObject, protocols=[MyProto]):
+                def testMethod2_(self, x):
+                    pass
+
+                @objc.typedSelector(b"@@:")
+                def testMethod(self):
+                    pass
+
+        with self.assertRaisesRegex(objc.error, "^ProtoClass2$"):
+            objc.lookUpClass("ProtoClass2")
+
+        for cls in objc.getClassList():
+            self.assertNotEqual(cls.__name__, "ProtoClass2")
+
+    def test_cleanup_protocol(self):
+        SomeProto = objc.informal_protocol(
+            "SomeProto",
+            (
+                objc.selector(
+                    None, selector=b"cleanupMethod", signature=b"I@:", isRequired=1
+                ),
+            ),
+        )
+
+        class ClassImplementingSomeProto1(NSObject):
+            def cleanupMethod(self):
+                return 1
+
+        self.assertResultHasType(ClassImplementingSomeProto1.cleanupMethod, b"I")
+
+        # XXX: This won't cause GC for the value due to how
+        #      the bridge is implemented...
+        del SomeProto
+
+        class ClassImplementingSomeProto2(NSObject):
+            def cleanupMethod(self):
+                return 1
+
+        self.assertResultHasType(ClassImplementingSomeProto2.cleanupMethod, b"I")
+
 
 EmptyProtocol = objc.formal_protocol("EmptyProtocol", None, ())
 
@@ -96,6 +173,9 @@ class TestFormalOCProtocols(TestCase):
             (b"method2:", b"v@:i"),
         )
 
+        with self.assertRaisesRegex(ValueError, "depythonifying 'SEL', got 'int'"):
+            OC_TestProtocol.descriptionForInstanceMethod_(42)
+
     def testImplementFormalProtocol(self):
         class MyClassNotImplementingProtocol(NSObject):
             pass
@@ -269,11 +349,10 @@ class TestFormalProtocols(TestCase):
             def anAnotherOne_(self, a):
                 pass
 
+            @classmethod
             def aClassOne_(self, a):
                 pass
 
-            aClassOne_ = classmethod(aClassOne_)
-
         self.assertEqual(ImplementingMyClassProtocol.anAnotherOne_.signature, b"i@:i")
         self.assertEqual(ImplementingMyClassProtocol.aClassOne_.isClassMethod, True)
         self.assertEqual(ImplementingMyClassProtocol.aClassOne_.signature, b"@@:i")
@@ -358,14 +437,42 @@ class TestFormalProtocols(TestCase):
             MyClassProtocol.descriptionForClassMethod_(b"nosuchmethod"), None
         )
 
-    def dont_testObjCInterface(self):
-        # TODO: tests that access the Objective-C interface of protocols
-        # (those methods should be forwarded to the underlying object, as
-        #  with objc.pyobjc_unicode).
-        # NOTE: This is not very important, the only methods that are not
-        # explicitly wrapped should be compatibility methods that will
-        # cause a warning when called.
-        self.assertEqual(1, 0)
+        with self.assertRaisesRegex(ValueError, "depythonifying 'SEL', got 'int'"):
+            MyClassProtocol.descriptionForClassMethod_(42)
+
+    def test_method_of_wrong_kind_instance(self):
+        class ClassWithClassMethodBase1(NSObject):
+            @classmethod
+            def protoMethod(self):
+                return 42
+
+        with self.assertRaisesRegex(
+            TypeError,
+            "does not correctly implement protocol MyProtocol: method 'protoMethod' is not an instance method",
+        ):
+
+            class ClassWithClassMethod(
+                ClassWithClassMethodBase1, protocols=[MyProtocol]
+            ):
+                def anotherProto_with_(self, a, b):
+                    pass
+
+    def test_method_of_wrong_kind_class(self):
+        class ClassWithClassMethodBase2(NSObject):
+            @objc.typedSelector(b"@@:i")
+            def aClassOne_(self, a):
+                return a
+
+        with self.assertRaisesRegex(
+            TypeError,
+            "does not correctly implement protocol MyProtocol: method 'aClassOne:' is not a class method",
+        ):
+
+            class ClassWithInstanceMethod(
+                ClassWithClassMethodBase2, protocols=[MyClassProtocol]
+            ):
+                def anAnotherOne_(self, a):
+                    return a
 
 
 class Test3InformalProtocols(TestCase):
@@ -401,6 +508,16 @@ MyClassProtocol3 = objc.formal_protocol(
     ],
 )
 
+MyProtocol4 = objc.formal_protocol(
+    "MyProtocol4",
+    None,
+    (
+        objc.selector(
+            None, selector=b"proto4Method", signature=b"I@:", isRequired=False
+        ),
+    ),
+)
+
 
 class TestFormalOCProtocols2(TestCase):
     def testImplementFormalProtocol(self):
@@ -512,6 +629,24 @@ class TestFormalProtocols2(TestCase):
             )
         )
 
+    def test_inherit_with_optional(self):
+
+        # First a class that doesn't implemented the protocol
+        # method. Is OK because the method is optional
+        class ImplementingProto4(NSObject, protocols=[MyProtocol4]):
+            pass
+
+        # Then a class that does implemented the protocol method,
+        # which affects the type signature.
+        class ImplementingProto4A(NSObject, protocols=[MyProtocol4]):
+            def proto4Method(self):
+                return 1
+
+        self.assertTrue(ImplementingProto4.conformsToProtocol_(MyProtocol4))
+        self.assertTrue(ImplementingProto4A.conformsToProtocol_(MyProtocol4))
+
+        self.assertResultHasType(ImplementingProto4A.proto4Method, b"I")
+
     def testImplementAnotherObject(self):
         anObject = NSObject.alloc().init()
 
@@ -624,7 +759,7 @@ class TestFormalProtocols2(TestCase):
         self.assertTrue(v.conformsTo_(v))
 
         with self.assertRaisesRegex(
-            TypeError, "Expecting objc.formal_protocol, got instance of 'int'"
+            TypeError, "argument 1 must be objc.formal_protocol, not int"
         ):
             v.conformsTo_(42)
 
